<?php

namespace Drupal\ultimate_table_field\Element;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Url;

/**
 * Ultimate table trait.
 */
trait UltimateTableTrait {

  /**
   * Get table first item ever.
   */
  public static function getFirstItemEver($i, $j, $values) {
    $item = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['table-first-item-ever'],
      ],
    ];

    // @todo Potentially using any cell field type here not only text.
    $item['text'] = [
      '#type' => 'textfield',
      '#size' => 50,
      '#value' => $values['columns'][$j] ?? NULL,
    ];
    return $item;
  }

  /**
   * Get Controlled items (item that control entire row and entire column).
   */
  public static function getControlledItem($i, $j, $values, $type = 'row') {
    $item = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['table-controlled-item'],
      ],
    ];
    $item['remove'] = self::buildRemoveButton($type, $i, $j);
    $column_value = $values['columns'][$j] ?? NULL;
    $row_value = $values['rows'][$i - 1][$j] ?? NULL;
    $item['text'] = [
      '#type' => 'textfield',
      '#size' => 40,
      '#value' => $type === 'column' ? $column_value : $row_value,
    ];
    return $item;
  }

  /**
   * Build cell summary element.
   */
  public static function buildCellSummaryElement(&$element, $i, $j, $items_data, $cell_id, $ut_cell_field_manager, $allowed_types) {
    $element['table'][$i][$j]['container'] = [
      '#type' => 'details',
      '#title' => t('Summary'),
      '#attributes' => [
        'class' => [empty($items_data) ? 'js-hide' : ''],
      ],
      '#prefix' => "<div id=\"$cell_id\">",
      '#suffix' => '</div>',
    ];
    $query = [
      'items_data' => Json::encode($items_data),
      'op' => 'update',
      'row_index' => $i,
      'column_index' => $j,
      'allowed_types' => Json::encode($allowed_types),
    ];
    foreach ($items_data as $key => $item_data) {
      $plugin_id = $item_data['type'] ?? '';
      if (!$ut_cell_field_manager->hasDefinition($plugin_id)) {
        continue;
      }
      $definition = $ut_cell_field_manager->createInstance($plugin_id);
      $element['table'][$i][$j]['container'][$key] = [
        '#type' => 'fieldset',
        '#prefix' => "<div id=\"table-cell-container-$i-$j\">",
        '#suffix' => "</div>",
        '#attributes' => [
          'class' => ['table-cell-item-summary'],
        ],
      ];
      $element['table'][$i][$j]['container'][$key]['summary'] = $definition->generateSummary($item_data);
      $link_title = [
        '#markup' => '<span class="update-item-button"></span>',
      ];
      $query['item_index'] = $key;
      $element['table'][$i][$j]['container'][$key]['update'] = static::modalLinkBuilder($link_title, $query);
      $element['table'][$i][$j]['container'][$key]['remove'] = self::buildRemoveButton('item', $i, $j, $key);
    }
    // Modal link opener.
    $query['op'] = 'add';
    $element['table'][$i][$j]['open_modal'] = static::modalLinkBuilder(t('Add'), $query);
  }

  /**
   * Modal link builder.
   */
  protected static function modalLinkBuilder($title, $query) {
    $op = $query['op'] ?? '';
    return [
      '#type' => 'link',
      '#title' => $title,
      '#url' => Url::fromRoute('ultimate_table_field.open_modal_form', [], [
        'query' => $query,
      ]),
      '#attributes' => [
        'class' => [
          'use-ajax',
          $op === 'add' ? 'button' : '',
        ],
      ],
      '#prefix' => '<div class="table-cell-item-actions">',
      '#suffix' => '</div>',
    ];
  }

  /**
   * Build remove button element.
   */
  public static function buildRemoveButton($type, $i, $j, $k = 0) {
    $name = $type . "_remove:";
    if ($type === 'row') {
      $name .= $i;
    }
    if ($type === 'column') {
      $name .= $j;
    }
    if ($type === 'item') {
      $name .= $i . '_' . $j . '_' . $k;
    }
    $message = $type == 'row' ? t('Deleting row...') : NULL;
    $message = !$message && $type == 'column' ? t('Deleting column...') : $message;
    $message = !$message && $type == 'item' ? t('Deleting item...') : $message;
    return [
      '#type' => 'submit',
      '#value' => ' ',
      '#submit' => $type !== 'item' ? [[self::class, 'elementSubmitCallback']] : [],
      '#name' => $name,
      '#attributes' => [
        'class' => ['button', 'button--danger', 'table-remove-img', 'use-ajax'],
      ],
      '#ajax' => [
        'callback' => [self::class, 'updateTable'],
        'wrapper' => 'ultimate-table',
        'method' => 'replace',
        'progress' => [
          'type' => 'throbber',
          'message' => $message,
        ],
      ],
    ];
  }

}
